import re
from datetime import date, datetime, timedelta

from pydantic import BaseModel, Field, field_validator
from libretificacaotjcore.enums.e_fase_retificacao import EFaseRetificacao


class SolicitacaoDTO(BaseModel):
    SolicitacaoId: int = Field(..., description="ID da solicitação")
    Cnpj: str = Field(..., description="CNPJ da empresa")
    DataInicio: date = Field(..., description="Data de início no formato YYYY-MM-DD")
    DataFim: date = Field(..., description="Data de fim no formato YYYY-MM-DD")
    CertificadoId: int = Field(..., description="ID do certificado")
    Fase: int | None = Field(None, description="fase de retificação")

    @field_validator("SolicitacaoId")
    @classmethod
    def validar_solicitacao_id(cls, v: int) -> int:
        if v <= 0:
            raise ValueError("O solicitacaoId deve ser um inteiro positivo.")
        return v

    @field_validator("CertificadoId")
    @classmethod
    def validar_certificado_id(cls, v: int) -> int:
        if v <= 0:
            raise ValueError("O certificadoId deve ser um inteiro positivo.")
        return v

    @field_validator("Cnpj")
    @classmethod
    def validar_cnpj(cls, v: str) -> str:
        cnpj_limpo = re.sub(r"\D", "", v)
        if len(cnpj_limpo) != 14 or not cnpj_limpo.isdigit():
            raise ValueError("O CNPJ deve conter 14 dígitos numéricos.")
        return cnpj_limpo

    @field_validator("DataInicio", mode="before")
    @classmethod
    def formatar_data_inicio(cls, value: str) -> date:
        try:
            return datetime.strptime(value, "%Y-%m-%d").date()
        except ValueError as ve:
            raise ValueError("A dataInicio deve estar no formato YYYY-MM-DD.") from ve

    @field_validator("DataFim", mode="before")
    @classmethod
    def ajustar_data_fim(cls, value: str) -> date:
        ano, mes = map(int, value.split("-")[:2])
        if mes < 1 or mes > 12:
            raise ValueError(
                "A dataFim deve estar no formato YYYY-MM-DD e conter um mês válido."
            )
        if mes == 12:
            proximo_mes = datetime(ano + 1, 1, 1)
        else:
            proximo_mes = datetime(ano, mes + 1, 1)
        return (proximo_mes - timedelta(days=1)).date()
    
    @field_validator("Fase")
    @classmethod
    def validar_fase(cls, v: str) -> str | None:
        if v is None:
            return v
        
        if v not in EFaseRetificacao:
            raise ValueError(f"Fase '{v}' não é válido.")
        return v



import aio_pika
import json
from aio_pika.abc import AbstractRobustConnection, AbstractChannel, AbstractQueue

class RabbitMQPublisher:
    def __init__(
        self,
        host: str,
        queue: str,
        username: str,
        password: str,
        vhost: str = "/",
    ):
        self.host = host
        self.queue = queue
        self.username = username
        self.password = password
        self.vhost = vhost

        self.connection: AbstractRobustConnection | None = None
        self.channel: AbstractChannel | None = None
        self.queue_obj: AbstractQueue | None = None

    async def connect(self):
        self.connection = await aio_pika.connect_robust(
            host=self.host,
            login=self.username,
            password=self.password,
            virtualhost=self.vhost,
            heartbeat=60,
        )

        
        self.channel = await self.connection.channel()
        # garante que mensagens não são perdidas
        self.queue_obj = await self.channel.declare_queue(
            self.queue, durable=True
        )

    async def publish(self, message: dict):
        if not self.channel:
            raise RuntimeError("❌ Canal RabbitMQ não conectado. Chame connect() antes.")

        body = json.dumps(message).encode("utf-8")
        await self.channel.default_exchange.publish(
            aio_pika.Message(
                body=body,
                delivery_mode=aio_pika.DeliveryMode.PERSISTENT,  # torna a msg persistente
            ),
            routing_key=self.queue,
        )
        print(f"✅ Mensagem publicada na fila '{self.queue}': {message}")

    async def close(self):
        if self.connection:
            await self.connection.close()




from pymongo.errors import BulkWriteError

class ArquivoRepository:
    def __init__(self, db):
        self.__db = db

    async def inserir_arquivo(self, arquivo: dict) -> bool:
        
        try:
            arquivo_no_db = await self.__db.arquivos.find_one(
                {"solicitacaoId": arquivo["solicitacaoId"], "cpf": arquivo["cpf"]}
            )

            if arquivo_no_db is None:
                await self.__db.arquivos.insert_one(arquivo)
                return True

            await self.__db.arquivos.delete_one(
                {"solicitacaoId": arquivo["solicitacaoId"], "cpf": arquivo["cpf"]}
            )
            await self.__db.arquivos.insert_one(arquivo)
            return True
        except Exception as e:
            print(f"❌ Erro ao inserir o arquivo: {e}")
            return False
        
    async def inserir_arquivos_em_lote(self, arquivos: list[dict]) -> bool:
        try:
            if not arquivos:
                return False

            filtros = [{"solicitacaoId": a["solicitacaoId"], "cpf": a["cpf"]} for a in arquivos]
            await self.__db.arquivos.delete_many({"$or": filtros})

            await self.__db.arquivos.insert_many(arquivos)
            return True
        except BulkWriteError as bwe:
            print(f"❌ Erro de escrita em lote: {bwe.details}")
            return False
        except Exception as e:
            print(f"❌ Erro ao inserir arquivos em lote: {e}")
            return False

    async def remover_arquivo(self, solicitacaoId: int) -> bool:
        try:
            await self.__db.arquivos.delete_many({"solicitacaoId": solicitacaoId})
            return True
        except Exception as e:
            print(f"❌ Erro ao remover o arquivo: {e}")
            return False

    async def buscar_por_solicitacao_id(self, solicitacaoId: int) -> list[dict]:
        try:
            return await self.__db.arquivos.find({"solicitacaoId": solicitacaoId}).to_list(length=None)
        except Exception as e:
            print(f"❌ Erro ao buscar por solicitacaoId: {e}")
            return []